@@ -1,18 +1,41 @@
 package Dependency::Resolver;
-$Dependency::Resolver::VERSION = '0.04';
+$Dependency::Resolver::VERSION = '0.05';
 use utf8;
 use Moose;
 
+has modules => (
+               is       => "rw",
+               isa      => "HashRef",
+               default  => sub { {} },
+           );
+
+has addr => (
+               is       => "rw",
+               isa      => "HashRef",
+               default  => sub { {} },
+           );
+
+sub add {
+    my ($self, @nodes) = @_;
+
+    foreach my $n (@nodes){
+        push(@{$self->modules->{$n->{name}}}, $n);
+        $self->addr->{"$n"} = $n;
+    }
+    return 1;
+}
+
 sub dep_resolv {
     my ($self, $node, $resolved, $seen) = @_;
 
     $resolved ||= [];
     $seen     ||= [];
 
-    #print $node->name . "\n";
     push( @$seen, $node);
+    for my $dep_version ( @{$node->{deps}} ) {
+
+        my $dep = $self->search_best_dep($dep_version);
 
-    for my $dep ( @{$node->deps} ) {
         # if dep is not in resolved
         if ( ! grep { $_ eq $dep} @$resolved ) {
 
@@ -27,11 +50,75 @@ sub dep_resolv {
 }
 
 
+sub parse_module_args {
+    my($self, $module) = @_;
+
+    $module =~ s/\s+//g;
+    $module =~ m/^([A-Za-z0-9_:]+)([\s!<>=]+)(.*)$/;
+    my ($mod,$op,$ver) = ($1, $2, $3);
+    if ( ! defined $op ) { $mod = $module, $op = '>=', $ver = 0};
+    return ($mod,$op,$ver);
+}
+
+sub search_best_dep{
+    my($self, $dep_args) = @_;
+
+    # ex: dep_args : B >1, B<=3
+    my $result = [];
+    foreach my $dep ( split(/,/, $dep_args)){
+
+        my($mod,$op,$ver) = $self->parse_module_args($dep);
+        my $modules = $self->get_modules($mod, $op, $ver);
+
+        my %count = ();
+        $count{$_}++
+            for (@$result, @$modules);
+
+        my @intersection  = grep { $count{$_} == 2 } keys %count;
+
+        if( $result->[0] ){
+            $result = [ $self->_addr_to_mod(@intersection) ];
+        }
+        else{  $result = $modules}
+    }
+
+    $result = $self->_sort_by_version($result);
+    die "Module $dep_args non found ! " if ( ! defined $result->[-1] );
+    # returns the highest version
+    return $result->[-1];
+}
+
+
+sub _sort_by_version {
+    my ( $self, $nodes ) = @_;
+
+    return [ sort { $a->{version} cmp $b->{version} }  @$nodes ];
+}
+
+sub _addr_to_mod {
+    my ( $self,@addrs ) = @_;
+
+    return map {  $self->addr->{$_} } @addrs;
+}
+
+
+sub get_modules{
+    my($self, $mod, $op , $ver) = @_;
+    my $modules = [];
+    foreach my $m (@{$self->modules->{$mod}}){
+        my $mver = $m->{version};
+
+        if ( eval "$mver $op $ver"  ){
+            push(@$modules, $m);
+        }
+    }
+    return $modules;
+}
 
 
 =head1 NAME
 
-Dependency::Resolver - Dependency resolving algorithm
+Dependency::Resolver - Simple Dependency resolving algorithm
 
 =head1 VERSION
 
@@ -41,22 +128,33 @@ based on http://www.electricmonk.nl/log/2008/08/07/dependency-resolving-algorith
 
 =head1 SYNOPSIS
 
-my $a = Dependency::Resolver::Node->new( name => 'a' );
-my $b = Dependency::Resolver::Node->new( name => 'b' );
-my $c = Dependency::Resolver::Node->new( name => 'c' );
-my $d = Dependency::Resolver::Node->new( name => 'd' );
-my $e = Dependency::Resolver::Node->new( name => 'e' );
-
-$a->addDep($b);    # a depends on b
-$a->addDep($d);    # a depends on d
-$b->addDep($c);    # b depends on c
-$b->addDep($e);    # b depends on e
-$c->addDep($d);    # c depends on d
-$c->addDep($e);    # c depends on e
+my $a  = { name => 'A' , version => 1, deps => [ 'B == 1', 'D']};
+my $a2 = { name => 'A' , version => 2, deps => [ 'B => 2, B < 3', 'D']};
+my $b1 = { name => 'B' , version => 1, deps => [ 'C == 1',  'E'] };
+my $b2 = { name => 'B' , version => 2, deps => [ 'C == 2',  'E'] };
+my $b3 = { name => 'B' , version => 3, deps => [ 'C == 3',  'E'] };
+my $c1 = { name => 'C' , version => 1, deps => [ 'D', 'E'] };
+my $c2 = { name => 'C' , version => 2, deps => [ 'D', 'E'] };
+my $c3 = { name => 'C' , version => 3, deps => [ 'D', 'E'] };
+my $d  = { name => 'D' , version => 1};
+my $e  = { name => 'E' , version => 1};
 
 my $resolver = Dependency::Resolver->new;
 
 my $resolved = $resolver->dep_resolv($a);
+#  return [ $d, $e, $c1, $b1, $a ]
+
+$resolved = $resolver->dep_resolv($a2);
+# return [ $d, $e, $c2, $b2, $a2 ]
+
+# method used by dep_resolv (get_module, search_best_dep)
+$resolver->get_modules('B', '==', 1); # return  [$b1]
+$resolver->get_modules('B', '<=', 2); # return  [$b1, $b2]
+$resolver->get_modules('B', '>=', 1); # return  [$b1, $b2, $b3]
+
+$resolver->search_best_dep('B >= 1');       # return $b3 (highest version)
+$resolver->search_best_dep('B >= 1, B!=3');       # return $b2
+$resolver->search_best_dep('B >= 1, B<=3, B!=3'); # return $b2
 
 
 =head1 SUBROUTINES/METHODS
